using System;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Starksoft.Cryptography.OpenPGP;
using System.IO;

namespace OpenPGP_Sample
{
    public partial class Form1 : Form
    {
        private GnuPG m_gpg;
        private MemoryStream m_unencrypted;
        private MemoryStream m_encryptedOrSigned;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // create a new gnupg object and make it a local class variable
            m_gpg = new GnuPG();
            
            // register to receive the encrypt async event
            m_gpg.EncryptAsyncCompleted += new EventHandler<EncryptAsyncCompletedEventArgs>(m_gpg_EncryptAsyncCompleted);
            // register to receive the decrypt async event
            m_gpg.DecryptAsyncCompleted += new EventHandler<DecryptAsyncCompletedEventArgs>(m_gpg_DecryptAsyncCompleted);
            // register to receive the sign async event
            m_gpg.SignAsyncCompleted += new EventHandler<SignAsyncCompletedEventArgs>(m_gpg_SignAsyncCompleted);

            // load the registered OpenPGP keys in your local key ring to the GridView on the Windows Form
            LoadKeys();

            // set some default data to encrypt
            tbUnencrypted.Text = @"The Starksoft OpenPGP library provides classes that interface with the GPG.EXE command line tool.  The Starksoft OpenPGP libraries allows any .NET application to use GPG.EXE to encrypt or decypt data using .NET IO Streams.  No temporary files are required and everything is handled through streams.  Any .NET Stream object can be used as long as the source stream can be read and the destination stream can be written to.  But, in order for the Starksoft OpenPGP library to work you must first install the lastest version of GnuPG which includes GPG.EXE.  You can obtain the latest version at http://www.gnupg.org/.  See the GPG.EXE tool documentation for information on how to add keys to the GPG key ring and creating your public and private keys.";
        }

        // this method will load the GnuPG OpenPGP keys that are in your local key ring
        // into a GridView control on the Windows Form.
        private void LoadKeys()
        {
            try
            {
                // optionally you can specify the path to the binary gpg.exe program.
                // if the path is not specified the Starksoft OpenPGP library will locate the
                // gpg.exe program using the Windows registry.
                //gpg.BinaryPath = @"C:\Program Files\gnu\GnuPG\gpg.exe";

                // retrieve a collection of keys from the gpg.exe application and convert
                // that collection to a DataTable so we can easily bind it to a GridView control
                gvKeys.DataSource = m_gpg.GetKeys().ToDataTable();
            }
            catch
            {
                MessageBox.Show("An error occurred while trying access the OpenPGP security keys.\r\n\r\nPlease make sure you have installed Gnu's OpenPGP application\r\nand that the OpenPGP settings are correctly entered.\r\nDownload GnuPG from http:\\www.gnupg.org\r\n\r\nMake sure that you have created a private key using the GPG.EXE application.", "Starksoft OpenPGP Library Sample");
            }
        }

        // retrieve the recipient key that is selected by the user in the GridView control named gvKeys
        private string GetRecipientKey()
        {
            if (gvKeys.SelectedRows.Count == 0)
            {
                MessageBox.Show("Please select a recipient from your key ring.", "Encrypt");
                return "";
            }

            // retrieve the recipient UserId text data that is currently selected in the gvKeys control
            return gvKeys.SelectedRows[0].Cells["UserId"].Value.ToString();
        }


        // this method will encrypt any OpenPGP data found in the TextBox named tbUnencrypted to the TextBox control named tbEncrypted
        private void btnEncryptAsync_Click(object sender, EventArgs e)
        {
            // check to see if the gnupg object is busy
            if (m_gpg.IsBusy)
            {
                MessageBox.Show("The GnuPG object is currently busy doing another asynchronous action.");
                return;
            }

            // get the select key from the key GridView control
            string recipient = GetRecipientKey();
            
            // if no recipient is specified then don't do the encryption
            if (recipient.Length == 0)
                return;

            // create two memory stream object instances - one to hold the unencrypted data and the other 
            // stream holds the encrypted data.  We can use any System.IO stream such as 
            // FileStream but for this demo we will use a memory stream
            m_unencrypted = new MemoryStream(Encoding.ASCII.GetBytes(tbUnencrypted.Text));
            m_encryptedOrSigned = new MemoryStream();

            // specify a recipient that is in your key ring
            m_gpg.Recipient = recipient;

            // encrypt the data using IO Streams - any type of input and output IO Stream can be used
            // as long as the source (input) stream can be read and the destination (output) stream 
            // can be written to.  When the asychronous action is completed the EncryptAsyncCompleted event
            // will be invoked.  Your code will continue to execute and the encryption process will run 
            // in the background on another thread.
            m_gpg.EncryptAsync(m_unencrypted, m_encryptedOrSigned);

        }

        // this event is fired when the EncryptAsync method is complete.
        // examine the event arguments to find out if an error occurred or if the action
        // was cancelled
        private void m_gpg_EncryptAsyncCompleted(object sender, EncryptAsyncCompletedEventArgs e)
        {
            // display the error message if one occurred
            if (e.Error != null)
            {
                MessageBox.Show(e.Error.Message, "Encryption Error");
                return;
            }

            // create a stream reader so we can read all the data from the memory stream
            StreamReader reader = new StreamReader(m_encryptedOrSigned);

            // set the encrypted data stream to the starting position so we can read the entire contents
            m_encryptedOrSigned.Position = 0;

            // read the text data from the memory stream.
            tbEncryptedOrSigned.Text = reader.ReadToEnd();
            
        }
                
        // this method will decrypt any OpenPGP data found in the TextBox named tbEncrypted to the TextBox control named tbUnencrypted
        private void btnDecryptAsync_Click(object sender, EventArgs e)
        {
            // check to see if the gnupg object is busy
            if (m_gpg.IsBusy)
            {
                MessageBox.Show("The GnuPG object is currently busy doing another asynchronous action.");
                return;
            }
            
            // create two memory stream - one to hold the unencrypted data and the other 
            // stream holds the encrypted data.  We can use any System.IO stream such as 
            // FileStream but for this demo we will use a memory stream
            m_unencrypted = new MemoryStream();
            m_encryptedOrSigned = new MemoryStream(Encoding.ASCII.GetBytes(tbEncryptedOrSigned.Text));
                      
            // specify our secret passphrase (if you have one)
            m_gpg.Passphrase = tbPassphrase.Text;

            // decrypt the data using IO Streams - any type of input and output IO Stream can be used
            // as long as the source (input) stream can be read and the destination (output) stream 
            // can be written to.  When the asychronous action is completed the DecryptAsyncCompleted event
            // will be invoked.  Your code will continue to execute and the decryption process will run 
            // in the background on another thread.
            m_gpg.DecryptAsync(m_encryptedOrSigned, m_unencrypted);
        }

        // this event is fired when the DecryptAsync method is complete.
        // examine the event arguments to find out if an error occurred or if the action
        // was cancelled
        private void m_gpg_DecryptAsyncCompleted(object sender, DecryptAsyncCompletedEventArgs e)
        {
            // test to see if an error occurred
            if (e.Error != null)
            {
                // see if the error was of a special type that tells us the passphrase entered is bad
                if (e.Error.GetType() == typeof(GnuPGBadPassphraseException))
                {
                    MessageBox.Show("Please enter a valid passphrase.", "Passphrase Needed");
                    tbPassphrase.Focus();
                    return;
                }
                else
                {
                    MessageBox.Show(e.Error.Message, "Decryption Error");
                    return;
                }
            }

            // create a stream reader so we can read all the data from the memory stream
            StreamReader reader = new StreamReader(m_unencrypted);

            // set the encrypted data stream to the starting position so we can read the entire contents
            m_unencrypted.Position = 0;

            // read the text data from the memory stream.
            tbUnencrypted.Text = reader.ReadToEnd();

        }
        
        // clear any data in the TextBox control named tbUnencrypted
        private void lnkClear_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            tbUnencrypted.Text = "";
        }

        // clear any data in the TextBox control named tbEncrypted
        private void lnkClearEncrypted_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            tbEncryptedOrSigned.Text = "";
        }

        private void btnSignAsync_Click(object sender, EventArgs e)
        {
            // check to see if the gnupg object is busy
            if (m_gpg.IsBusy)
            {
                MessageBox.Show("The GnuPG object is currently busy doing another asynchronous action.");
                return;
            }

            // create two memory stream - one to hold the unencrypted data and the other 
            // stream holds the encrypted data.  We can use any System.IO stream such as 
            // FileStream but for this demo we will use a memory stream
            m_unencrypted = new MemoryStream(Encoding.ASCII.GetBytes(tbUnencrypted.Text));
            m_encryptedOrSigned = new MemoryStream();

            // specify our secret passphrase (if you have one)
            m_gpg.Passphrase = tbPassphrase.Text;

            // sign the data using IO Streams - any type of input and output IO Stream can be used
            // as long as the source (input) stream can be read and the destination (output) stream 
            // can be written to.  When the asychronous action is completed the SignAsyncCompleted event
            // will be invoked.  Your code will continue to execute and the decryption process will run 
            // in the background on another thread.
            m_gpg.SignAsync(m_unencrypted, m_encryptedOrSigned);

        }

        private void m_gpg_SignAsyncCompleted(object sender, SignAsyncCompletedEventArgs e)
        {
            // test to see if an error occurred
            if (e.Error != null)
            {
                // see if the error was of a special type that tells us the passphrase entered is bad
                if (e.Error.GetType() == typeof(GnuPGBadPassphraseException))
                {
                    MessageBox.Show("Please enter a valid passphrase.", "Passphrase Needed");
                    tbPassphrase.Focus();
                    return;
                }
                else
                {
                    MessageBox.Show(e.Error.Message, "Signing Error");
                    return;
                }
            }

            // create a stream reader so we can read all the data from the memory stream
            StreamReader signed = new StreamReader(m_encryptedOrSigned);

            // set the encrypted data stream to the starting position so we can read the entire contents
            m_encryptedOrSigned.Position = 0;

            // read the text data from the memory stream.
            tbEncryptedOrSigned.Text = signed.ReadToEnd();

        }

    }
}